#include <stdio.h>
#include <stdlib.h>
#include <regex.h>
#include "sicstus/sicstus.h"

regex_t *regcomp_pl(const char *s1,
		    const int reg_extended,
		    const int reg_newline,
		    const int reg_icase,
		    const int reg_nosub) 
{
  regex_t r;
  int err;
  int flags=0;
  regex_t *t = (regex_t *)malloc(sizeof(r));
  if (reg_extended) flags=flags|REG_EXTENDED;
  if (reg_newline)  flags=flags|REG_NEWLINE;
  if (reg_icase)    flags=flags|REG_ICASE;
  if (reg_nosub)    flags=flags|REG_NOSUB;

  err = regcomp (t, s1, flags);
  if (err)
    { 
      SP_term_ref sp_err=SP_new_term_ref();
      char errmsg[100];
      regerror (err, t, errmsg, 100);
      puts (errmsg);
      regfree (t);
      free (t);
      SP_put_string(sp_err,"Error in regular expression");
      SP_raise_exception(sp_err);
    } 
  else
    return t;
}

void regfree_pl(regex_t *r) {
  regfree(r);
  free(r);
}

SP_term_ref regexec_pl(char *s2,             /* string to be matched    */
		       const int reg_notbol, 
		       const int reg_noteol,
		       regex_t *r,           /* compiled expression     */
		       const int need_subs   /* return matched subs ?   
					      * should be 0 if r compiled
					      * with REG_NOSUB
					      */
		       ){
  SP_term_ref t=SP_new_term_ref();
  int err;
  int flags=0;
  int subs=(*r).re_nsub;
  regmatch_t regs[subs+1];
  char errmsg[100];
  
  if (reg_notbol) flags=flags|REG_NOTBOL;
  if (reg_noteol) flags=flags|REG_NOTEOL;
  err = regexec (r, s2, subs+1, regs, flags);
  switch (err) {
  case 0:
    {
      SP_term_ref head=SP_new_term_ref();
      if (need_subs) {
	int i;
	for (i=subs;i>-1;i--){
	  /* it's easier to construct the list from right-to-left */
	  SP_put_list_n_chars(head,SP_new_term_ref(),
			      regs[i].rm_eo-regs[i].rm_so,
			      &s2[regs[i].rm_so]);
	  SP_cons_list(t,head,t);
	}
      }
      else {
	SP_put_list_chars(head,SP_new_term_ref(),"yes");
	SP_cons_list(t,head,SP_new_term_ref());
      }
      return t;
    }

  case REG_NOMATCH:
    {
      return t;
    }
    
  default:
    {
      regerror (err, r, errmsg, 100);
      SP_put_string(t,"Error in regular expression");
      SP_raise_exception(t);
    }
  }
}




